"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.login = exports.pwHash = exports.logout = exports.fromChannel = exports.req = exports.unsubEntityUpdates = exports.subEntityUpdates = exports.unsub = exports.sub = exports.pub = void 0;

var _extendedDsClient = _interopRequireWildcard(require("extended-ds-client"));

var _sha = _interopRequireDefault(require("crypto-js/sha256"));

var R = _interopRequireWildcard(require("ramda"));

var _jstiip = _interopRequireDefault(require("jstiip"));

var _subSeconds = _interopRequireDefault(require("date-fns/fp/subSeconds"));

var _dmUtils = require("./dm-utils");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) { var desc = Object.defineProperty && Object.getOwnPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : {}; if (desc.get || desc.set) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } } newObj["default"] = obj; return newObj; } }

function asyncGeneratorStep(gen, resolve, reject, _next, _throw, key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { Promise.resolve(value).then(_next, _throw); } }

function _asyncToGenerator(fn) { return function () { var self = this, args = arguments; return new Promise(function (resolve, reject) { var gen = fn.apply(self, args); function _next(value) { asyncGeneratorStep(gen, resolve, reject, _next, _throw, "next", value); } function _throw(err) { asyncGeneratorStep(gen, resolve, reject, _next, _throw, "throw", err); } _next(undefined); }); }; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; var ownKeys = Object.keys(source); if (typeof Object.getOwnPropertySymbols === 'function') { ownKeys = ownKeys.concat(Object.getOwnPropertySymbols(source).filter(function (sym) { return Object.getOwnPropertyDescriptor(source, sym).enumerable; })); } ownKeys.forEach(function (key) { _defineProperty(target, key, source[key]); }); } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

/**
 * Publishes a message through the backend.
 * @param {string} channel - The channel to publish on.
 * @param {Array} [pl=[]] - The payload to send.
 * @param {string} [sig] - The signal data to apply to the published message.
 */
var pub = function pub(channel) {
  var pl = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : [];
  var sig = arguments.length > 2 ? arguments[2] : undefined;
  var msg = new _jstiip["default"]({
    pl: pl
  });
  if (sig) msg.sig = sig;
  var msgOut = msg.toJS();

  _extendedDsClient.ds.client.event.emit(channel, msgOut); // console.log('PUB:', channel, msgOut);

};
/**
 * Subscribes to a channel on the backend.
 * @param  {function} callback - The callback to trigger when a message arrives.
 * @param  {string} channel - The channel to subscribe to.
 */


exports.pub = pub;
var sub = R.curry(function (callback, channel) {
  return _extendedDsClient.ds.client.event.subscribe(channel, callback);
});
/**
 * Unsubscribes from a channel on the backend.
 * @param  {string} channel - The channel to unsubscribe from.
 * @param  {function} [callback] - The callback that was previously registered with subscribe. If
 *  the callback is omitted, all subscriptions for the channel will be removed.
 */

exports.sub = sub;

var unsub = function unsub(channel, callback) {
  return _extendedDsClient.ds.client.event.unsubscribe(channel, callback);
};

exports.unsub = unsub;
var confUpdatesChannel = R.curry(function (type, rid) {
  return "conf/".concat(type, "/update/").concat(rid);
});

var busChannel = function busChannel(rid) {
  return "data/".concat(rid);
};

var subEntityUpdates = function subEntityUpdates(type) {
  var ridToChannel = confUpdatesChannel(type);
  return function (cb) {
    return R.forEach(R.o(sub(cb), ridToChannel));
  };
};

exports.subEntityUpdates = subEntityUpdates;

var unsubEntityUpdates = function unsubEntityUpdates(type) {
  var ridToChannel = confUpdatesChannel(type);
  return function (rids, cb) {
    return R.forEach(R.o(function (ch) {
      return unsub(ch, cb);
    }, ridToChannel))(rids);
  };
};
/**
 * Performs a request to the backend.
 * @param  {string} id - The id of the rpc
 * @param  {Object} [args={}] - Arguments passed on to the rpc
 * @return {any} - Depends on the rpc.
 */
// TODO: Refactor temporarily to rids:* to target all, but only for 'conf...'-requests


exports.unsubEntityUpdates = unsubEntityUpdates;
var req = R.curry(function (id, args) {
  return _extendedDsClient.ds.client.rpc.p.make(id, args)["catch"](function (err) {
    console.error(id, args, err);
    throw err;
  });
});
exports.req = req;

var fromChannel = function fromChannel(_ref) {
  var rid = _ref.rid,
      last = _ref.last,
      tail = _ref.tail,
      ttl = _ref.ttl,
      cb = _ref.cb,
      noSub = _ref.noSub;

  if (last || tail) {
    req('data-history/read', _objectSpread({
      rid: rid
    }, ttl ? {
      filter: {
        ct: {
          $gte: (0, _dmUtils.dateToTs)((0, _subSeconds["default"])(ttl)(new Date()))
        }
      }
    } : {}, {
      limit: tail || Number(last)
    })).then(function (reply) {
      return reply.reverse().forEach(function (msg) {
        return cb(msg);
      });
    });
  }

  if (!noSub) {
    var channel = busChannel(rid);
    sub(cb, channel);
    return channel; // this.subscriptions = R.append(busChannel)(this.subscriptions);
  }

  return false;
}; // /**
//  * Converts a rid to be url-friendly
//  * @param {string} rid - The record id to convert.
//  * @return {string} - An url-friendly rid.
//  */
// export const ridToUrl = R.compose(
//   R.replace(':', '-'),
//   R.replace('#', ''),
// );
//
// /**
//  * Converts a url-friendly rid to an actual rid.
//  * @param {string} rid - The url-friendly record id to convert.
//  * @return {string} - An actual record id.
//  */
// export const urlToRid = R.compose(
//   rid => `#${rid}`,
//   R.replace('-', ':'),
// );


exports.fromChannel = fromChannel;

var logout = function logout() {
  if (!_extendedDsClient.ds.client) return;

  _extendedDsClient.ds.client.off();

  _extendedDsClient.ds.client.close();
};
/**
 * Creates a connection to the backend server.
 * @param {string} serverAddress
 * @param {function} onConnectionError - Callback triggered on connection errors.
 * @param {function} onUnexpectedError - Callback triggered on general errors in the client.
 * @param {}
 * @return {Promise} - [description]
 */
// TODO: What does this function return? client? as promise?


exports.logout = logout;

var createConnection =
/*#__PURE__*/
function () {
  var _ref2 = _asyncToGenerator(
  /*#__PURE__*/
  regeneratorRuntime.mark(function _callee(serverAddress, onConnectionError, onUnexpectedError) {
    return regeneratorRuntime.wrap(function _callee$(_context) {
      while (1) {
        switch (_context.prev = _context.next) {
          case 0:
            console.log('createConnection:', serverAddress, _extendedDsClient.ds.client);
            if (_extendedDsClient.ds.client) logout();
            _extendedDsClient.ds.client = (0, _extendedDsClient["default"])(serverAddress);

            _extendedDsClient.ds.client.on('error', onUnexpectedError);

          case 4:
          case "end":
            return _context.stop();
        }
      }
    }, _callee);
  }));

  return function createConnection(_x, _x2, _x3) {
    return _ref2.apply(this, arguments);
  };
}();

var pwHash = function pwHash(pw) {
  return String((0, _sha["default"])(pw));
};

exports.pwHash = pwHash;

var login =
/*#__PURE__*/
function () {
  var _ref3 = _asyncToGenerator(
  /*#__PURE__*/
  regeneratorRuntime.mark(function _callee2(serverAddress, id, password, onConnectionError, onUnexpectedError) {
    return regeneratorRuntime.wrap(function _callee2$(_context2) {
      while (1) {
        switch (_context2.prev = _context2.next) {
          case 0:
            createConnection(serverAddress, onConnectionError, onUnexpectedError);
            _context2.next = 3;
            return _extendedDsClient.ds.client.p.login({
              id: id,
              password: password
            });

          case 3:
            _extendedDsClient.ds.client.on('connectionStateChanged', function (state) {
              // We need to set this only when we know that we got a working logged in connection
              console.log('CONNECTION STATE:', state);
              if (state === 'ERROR' && _extendedDsClient.ds.client.connectionState !== 'ERROR') onConnectionError();
              _extendedDsClient.ds.client.connectionState = state;
            });

            return _context2.abrupt("return", {
              id: id,
              password: password,
              serverAddress: serverAddress
            });

          case 5:
          case "end":
            return _context2.stop();
        }
      }
    }, _callee2);
  }));

  return function login(_x4, _x5, _x6, _x7, _x8) {
    return _ref3.apply(this, arguments);
  };
}(); // const ensureRids = ensureKey('rids', []);
//
// export const fetchEntity = type => {
//   if (!fetchMapping[type]) throw new Error('No such entity type:', type);
//   return async (args = {}) => {
//     let records = [];
//     try {
//       records = await req(`conf/${fetchMapping[type]}`, ensureRids(args));
//     } catch (err) {
//       console.warn(`Fetching ${type}s failed:`, err);
//     }
//     return R.indexBy(R.prop('rid'), records);
//   };
// };


exports.login = login;